<?php

namespace Drupal\pdf_slideshow\Gateway;

use Drupal\Core\Entity\Entity;
use Drupal\Core\File\FileSystem;
use Drupal\Core\State\State;
use Drupal\pdf_slideshow\Manager\PdfSlideshowImagickManager;

/**
 * Class PdfSlideshowStorageGateway.
 *
 * @package Drupal\pdf_slideshow\Gateway
 */
class PdfSlideshowStorageGateway {

  /**
   * State.
   *
   * @var \Drupal\Core\State\State
   */
  protected $state;

  /**
   * File system.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * PdfSlidesshowImagickManager.
   *
   * @var \Drupal\pdf_slideshow\Manager\PdfSlideshowImagickManager
   */
  protected $pdfSlideshowImagickManager;

  /**
   * PdfSlidesshowEntityGateway.
   *
   * @var \Drupal\pdf_slideshow\Gateway\PdfSlideshowEntityGateway
   */
  protected $pdfSlideshowEntityGateway;

  /**
   * PdfSlideshowStorageGateway constructor.
   *
   * @param \Drupal\Core\State\State $state
   *   Sate.
   * @param \Drupal\Core\File\FileSystem $fileSystem
   *   Filesystem.
   * @param \Drupal\pdf_slideshow\Manager\PdfSlideshowImagickManager $pdfSlideshowImagickManager
   *   PdfSlideshowImagickManager.
   * @param \Drupal\pdf_slideshow\Gateway\PdfSlideshowEntityGateway $pdfSlideshowEntityGateway
   *   PdfSlideshowEntityGateway.
   */
  public function __construct(
    State $state,
    FileSystem $fileSystem,
    PdfSlideshowImagickManager $pdfSlideshowImagickManager,
    PdfSlideshowEntityGateway $pdfSlideshowEntityGateway
  ) {
    $this->state = $state;
    $this->fileSystem = $fileSystem;
    $this->pdfSlideshowImagickManager = $pdfSlideshowImagickManager;
    $this->pdfSlideshowEntityGateway = $pdfSlideshowEntityGateway;
  }

  /**
   * FetchPdfImage.
   *
   * @param int|string $fid
   *   Pdf file id.
   *
   * @return array|null
   *   Pdf image file ids.
   */
  public function fetchPdfImageIds($fid) {
    $values = $this->state->get('pdf_slideshow.' . $fid);
    if ($values) {
      return explode('_', $values);
    }
    return NULL;
  }

  /**
   * Persist images.
   *
   * @param \Drupal\Core\Entity\Entity $fileEntity
   *   File entity.
   * @param int|string $width
   *   Width.
   * @param int|string $height
   *   Height.
   * @param int|string $limit
   *   Limit.
   *
   * @return mixed
   *   File image urls.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \ImagickException
   */
  public function persistPdfImageIds(Entity $fileEntity, $width, $height, $limit) {
    if ($fileEntity && $fileEntity->getMimeType() == 'application/pdf') {
      $fileUri = $fileEntity->getFileUri();
      $fileUriArray = explode('/', $fileUri);
      $fileName = end($fileUriArray);
      $sourcePath = $this->fileSystem->realpath($fileUri);
      if ($sourcePath) {
        $imageUrls = $this->pdfSlideshowImagickManager->generateImage($sourcePath,
          $fileName . '.png', $width, $height, $limit);
        $fileEntityIds = $this->pdfSlideshowEntityGateway->createFileEntities($imageUrls);
        $this->state->set('pdf_slideshow.' . $fileEntity->id(),
          implode('_', $fileEntityIds));
        return $this->pdfSlideshowEntityGateway->getPdfFileImageUrls($fileEntityIds);
      }
    }
    return NULL;
  }

}
