<?php


namespace Drupal\pdf_slideshow\Manager;

use Drupal\Core\Render\Renderer;
use Drupal\pdf_slideshow\Gateway\PdfSlideshowEntityGateway;

/**
 * Class PdfSlideshowRenderManager
 *
 * @package Drupal\pdf_slideshow\Manager
 */
class PdfSlideshowRenderManager {

  /**
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * @var \Drupal\pdf_slideshow\Manager\PdfSlideshowImageManager
   */
  protected $pdfSlideShowImageManager;

  /**
   * @var \Drupal\pdf_slideshow\Gateway\PdfSlideshowEntityGateway
   */
  protected $pdfSlideshowEntityGateway;

  /**
   * PdfSlideshowRenderManager constructor.
   *
   * @param \Drupal\Core\Render\Renderer $renderer
   * @param \Drupal\pdf_slideshow\Manager\PdfSlideshowImageManager $pdfSlideShowImageManager
   * @param \Drupal\pdf_slideshow\Gateway\PdfSlideshowEntityGateway $pdfSlideshowEntityGateway
   */
  public function __construct(
    Renderer $renderer,
    PdfSlideshowImageManager $pdfSlideShowImageManager,
    PdfSlideshowEntityGateway $pdfSlideshowEntityGateway
  ) {
    $this->renderer = $renderer;
    $this->pdfSlideShowImageManager = $pdfSlideShowImageManager;
    $this->pdfSlideshowEntityGateway = $pdfSlideshowEntityGateway;
  }

  /**
   * HTML slideshow render.
   *
   * @param array $pdfFileIds
   * @param $width
   * @param $height
   * @param $limit
   *
   * @return \Drupal\Component\Render\MarkupInterface|string|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \ImagickException
   */
  public function renderHtmlSlideshow(
    array $pdfFileIds,
    $width,
    $height,
    $limit
  ) {

    $entities = $this->pdfSlideshowEntityGateway->getFileEntities($pdfFileIds);
    if ($entities) {
      $slidesArray = $this->pdfSlideShowImageManager->getPdfImageUrls($entities,
        $width,
        $height, $limit);

      $renderArray = [
        '#theme' => 'pdf_slideshow',
        '#slides' => $slidesArray,
        '#attached' => [
          'library' => [
            'pdf_slideshow/pdf-slideshow',
          ],
        ],
      ];
      return $this->renderer->render($renderArray);
    }
    return NULL;
  }

}
